.TH cabinet_open 3C "TACOMA 1.2" "C Primitives"
.SH NAME
cabinet_open \- opens a cabinet
.sp
cabinet_close \- closes a cabinet
.sp
cabinet_discard \- deletes a cabinet
.sp
cabinet_list \- list the folders of a cabinet
.SH SYNOPSIS
\fC#include <tacoma.h>\fR
.PP
\fCcabinet cabinet_open(char *name, int flags);\fR
.PP
\fCvoid cabinet_close(cabinet cab);\fR
.PP
\fCvoid cabinet_discard(cabinet cab);\fR
.PP
\fCchar **cabinet_list(cabinet cab);\fR
.SH DESCRIPTION
The current TACOMA cabinet implementation is a very basic one. It does
not support locking, access control, or other concurrency
mechanisms. It is not even clear at this time if the TACOMA cabinets
should be accessed directly or through special cabinet
agents. However it is currently modeled as close as possible to the
current TACOMA briefcase primitives. The only exception being
that cabinets may not be archived and shift/rotate operations can not
be performed directly on cabinet folders.

\fCcabinet_open()\fR
command opens the
.I name
cabinet. Currently, only one flag is supported, \fCTC_CREAT\fR, which
will create and initialize the cabinet if it does not exist. If the cabinet already exists this flag has no effect. 
Cabinets must be created using this flag before accessing them.
In the future, this
command may be extended to include locking.

\fCcabinet_close()\fR
command closes the cabinet
.IR cab .
In the case of future locking, it releases any locks held by the agent on that
cabinet.

\fCcabinet_discard()\fR
command deletes the cabinet
.I cab
and all folders associated with it. 
.SS Note
A cabinet is either closed or discarded after use, never both. Do not attempt to use both \fCcabinet_close()\fR and \fCcabinet_discard()\fR on the same cabinet.

\fCcabinet_list()\fR
command creates a list of all folders in the cabinet
.IR cab .
It returns this list as a NULL-terminated array of strings that each
represents the name of a folder in the cabinet. It is the
responsibility of the caller to free this array and the strings in
it. For this, \fCtacoma.h\fR defines a macro called \fCfree_copy(char
**)\fR which does exactly this.

.SH RETURN_VALUE
\fCcabinet_open()\fR
returns an allocated cabinet which should be deallocated by
\fCcabinet_close()\fR \fBor\fR \fCcabinet_discard()\fR.
\fCcabinet_list()\fR
returns a NULL-terminated array of strings or NULL if the cabinet does
not exist. 

.SH NOTE
In releases prior to Tacoma v1.2 there existed a function called \fCcabinet_create()\fR. This function has been removed from the current C implementation and replaced with a flag given to \fCcabinet_open()\fR  
.SH SEE ALSO
.IR folder_fetch (3C)
.SH AUTHOR
Kjell Irgens (kjelli@stud.cs.uit.no), Magne Rasmussen
(magner@stud.cs.uit.no)
and Nils Peter Sudmann (nilss@stud.cs.uit.no), University of
Tromsoe.
Concept by Dag Johansen (dag@cs.uit.no), Robbert van Renesse
(rvr@cs.cornell.edu) and Fred B. Schneider (fbs@cs.cornell.edu).
