'\"     # CS - begin code section
.de CS
.RS 4
\fC
.ps -1
.nf
.sp
..
'\"     # CE - end code section
.de CE
.fi
\fR
.ps +1
.RE
.sp .5
.. 
.TH meet 3C "TACOMA 1.1.3" "C Primitives"
.SH NAME
meet \- arrange a meeting with a (possibly) remote agent
.SH SYNOPSIS
\fC#include <meet.h>\fR
.PP
\fCint meet(char *agent, briefcase bc, int options);\fR
.SH DESCRIPTION
Prepares the supplied \fIbriefcase\fR and delivers it to a (possibly)
remote \fIagent\fR. The target \fIagent\fR must reside in the
\fC$TACOMAPATH/sysagents\fR directory.

The destination host is determined by rotating the HOST folder
of the supplied \fIbriefcase\fR, see
.IR folder_rotate (3C).
The first item of the HOST list is
the name of the remote host. A special host name of 'local' can be
used to meet with local agents.

If the result of this rotation is a zero length string (empty), meet
will try to extract a host name from the RETURNHOST folder. The
RETURNHOST folder is also used to send agents reporting fatal errors
back to the point of origin. This happens when a meet fails after the
briefcase has been delivered to the remote host. These agents report
errors by meeting
with ag_report, see
.IR ag_report (1).

.SS Folder inputs
.TP
HOST
a host or a list of hosts to visit. This folder will be rotated
once to the left by the meet operation.
.TP
RETURNHOST
a host to try to send the agent to if the HOST folder is
exhausted. It is also used to send the equivalence of an agent core
dump back to the point of origin should a meet fail.
.SH ""
The rotating of the HOST folder may be used to preset the path
of the \fIagent\fR through the network. The \fIagent\fR will visit
each site on the list, and will return to the host named in the
RETURNHOST folder when a meet operation encounters an empty item
on the HOST folder list. However, it is not always possible to
determine a full path. In this case, a new site could be added to the
HOST folder at each site (by using 
.IR folder_shift (3C)). 
However, it is important to
note that the host list should contain an empty item, to terminate the
\fIagent\fR's travel. The rotating of the HOST folder could
also be used to determine the actual path of the \fIagent\fR through
the network when it returns, since 
.IR folder_rotate (3C) 
never deletes items from a folder list.
.SS Options
.TP
TM_SYNC
This flag can be
used to perform a blocking RPC-style meeting. In this case, the agent
initiating the meet operation will be blocked until a
briefcase is returned. This means that the RETURNHOST
folder is ignored. Also, the supplied \fIbriefcase\fR will be
merged with the returned briefcase (the intersection between the two
data sets are taken from the returned briefcase). Synchronous meetings do not
cause error reports to be sent back to the host specified in
RETURNHOST. The reason is that agents performing synchronous meetings
always get a briefcase back, and may investigate this directly to
determine if an error has occurred.

Synchronous meetings should be used sparingly as they tie up resources
at the sites they travel through. At each site visited,
a process is created to deal exclusively with that
briefcase.
.TP
TM_LOCL
This flag forces a local meeting, e.g. the HOST folder will not be touched
by meet.
.TP
TM_NMRG
When specifying this option together with TM_SYNC, briefcases will not be
merged. If the activated agent returns a briefcase, it will replace the
\fIbriefcase\fR specified in the meet call. If TM_SYNC is not specified
this flag has no effect.
.TP
TM_IERR
Ignore errors when performing an asynchronous meeting, and do not create 
error reports. If TM_SYNC is
specified this flag has no effect.
.TP
TM_CRPT
Automatically encrypt the briefcase (using
.IR ag_crypt (1))
so that nobody except the receiving
firewall can decrypt it. This flag has no effect if the meeting is
performed locally.
.TP
TM_SIGN
Automatically sign the briefcase (using
.IR ag_crypt (1))
so that the receiving firewall may authenticate it. This flag has no effect if the meeting is
performed locally.
.TP
TM_PATH
The \fIagent\fR argument specifies a full path to the agent to be
activated. This switch is used by some system agents, and may be ignored by
meet or the receiving firewall when used by a client agent.
.TP
TM_NLOG
Using this option will disable logging of the meet. In certain cases it may be ignored
by meet or the receiving firewall. This option has no effect if the meet is performed
locally or synchronous (logging is not performed in these cases).
.SS Note
Even if possible, local agents should never be called directly
as a procedure call. Meet acts as a firewall against failures, and is
a transparent way of interacting with local agents without prior
knowledge of how they are implemented. Even if agents written as
Tcl libraries could be called directly from Tcl code, their failure
will cause the calling agent to fail unless errors are trapped with
the Tcl \fCcatch\fR command. Although most agents currently
are implemented as Tcl library routines, there is no guarantee that
this will not change in the future.

.SS Result
.TP
 0
Successful delivery of the \fIbriefcase\fR to the (remote) site. In
case of an RPC-style or local meeting, this also means the successful
return of a briefcase. The supplied \fIbriefcase\fR
has then been merged or replaced by the returned briefcase (as
requested by the TM_NMRG option).
.TP
1
The first site of the HOST folder could not be reached.
.TP
2
The site refused to accept the \fIbriefcase\fR.
.TP
3
No site could be found (the rotating of HOST folder returned an
empty item and RETURNHOST folder is empty).
.TP
4
Local \fIagent\fR terminated abnormally.
.TP
5
Local \fIagent\fR not found.
.TP
6
Some internal failure in the TACOMA system.
.TP
-x
Local \fIagent\fR returned error.
.SH ""

Agents should return a status of 0 indicating success or
negative value indicating failure. In case of a failure, the
agent may supply a further description in the STATUS
folder. The meaning of each agents negative return codes
should be described in the documentation of that agent.

.SH BUGS
In version 1.1 of Tacoma the TM_NMRG, TM_CRPT and TM_SIGN flags where not implemented.
.SH SEE ALSO
.IR tac_firewall (1), ag_tcl (1)

.SH AUTHOR
Nils Peter Sudmann (nilss@stud.cs.uit.no) and Magne Rasmussen
(magner@stud.cs.uit.no), University of Tromsoe. 
Concept by Dag Johansen (dag@cs.uit.no), Robbert van Renesse
(rvr@cs.cornell.edu) and Fred B. Schneider (fbs@cs.cornell.edu).
