/*******************************************************************************
 *
 * $Source: /users/nilss/source/RCS/meet.c,v $
 * $Header: meet.c,v 1.10 96/05/22 17:35:16 nilss Exp $
 * $Author: nilss $
 * $Date: 96/05/22 17:35:16 $
 * Language: C
 * Status: Experimental
 *
 * (c) Copyright 1996, TACOMA, all rights reserved.
 *
 ******************************************************************************/
#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#if HAVE_UNISTD_H
# include <sys/types.h>
# include <unistd.h>
#endif
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/stat.h>
#if HAVE_SYS_WAIT_H
# include <sys/wait.h>
#endif
#ifndef WEXITSTATUS
# define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
#endif
#ifndef WIFEXITED
# define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
#endif
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "tacoma.h"
#include "meet.h"
#include "secret.h"


/*
 * Use TM_DEBUG to debug meet()
 */
/*#define	TM_DEBUG*/

#ifndef PAGE_SIZE
# define PAGE_SIZE (1 << 12)
#endif /* PAGE_SIZE */
#ifndef SA_RESTART
# define SA_RESTART 0
#endif /* SA_RESTART */
#ifndef INADDR_NONE
#  define INADDR_NONE (-1)
#endif /* INADDR_NONE */
#ifndef MAXHOSTNAMELEN
#  define MAXHOSTNAMELEN 256
#endif /* MAXHOSTNAMELEN */

/*
 *	Utility functions used in this file.
 */
static int	local_meet( string, briefcase, int );
static void	tac_sigchld( int );
static void	tac_read_stderr( briefcase, string, string );
static void	tac_create_error_rep( briefcase );


int
meet( string agent, briefcase abc, int flags )
{
    int			port;
    int			retval;
    int			sockfd;
    char		status[TM_STAT_MSG_SIZE];
    string		arc;
    string		arcp;
    string		host;
    string		tacomaport;
    string*		data;
    cabinet		outlog;
    cabinet		syscab;
    briefcase		bc;
    unsigned long	inaddr;
    struct hostent*	hep;
    struct sockaddr_in	addr;

    /* meet local agent */
    if ( flags & TM_LOCL )
	return (local_meet(agent, abc, flags));

    /* find out which host to meet at */
    host = folder_rotate(abc, TAC_HOST, TAC_LEFT, 1);
    if ( host == NULL || *host == '\0' ) {
	free(host);
	host = folder_rotate(abc, TAC_RETURNHOST, TAC_LEFT, 1);
	if ( host == NULL || *host == '\0' ) {
	    folder_store(abc, TAC_STATUS, "No host could be found.", NULL);
	    free(host);
	    return (TMR_NOSITE);
	}
    }

    /* if local host, meet locally */
    if ( !strcmp(host, "local") || !strcmp(host, "localhost") ) {
	free(host);
	return (local_meet(agent, abc, flags));
    }

#ifdef TM_DEBUG
    fprintf(stderr, "Remote %s meet with '%s' on host '%s'.\n",
	    flags&TM_SYNC ? "sync" : "async", agent, host);
#endif

    folder_shift(abc, TAC_CONTACT, TAC_RIGHT, 0, agent, NULL);

    /* find and check port number */
    if ( (tacomaport = getenv(TAC_ENV_PORT)) == NULL ) {
	folder_store(abc, TAC_STATUS, "Environment variable '" TAC_ENV_PORT \
		     "' not set.", NULL);
	free(host);
	return (TMR_INTERNAL);
    }
    if ( isdigit(*tacomaport) )
	port = atoi(tacomaport);
    else {
	folder_store(abc, TAC_STATUS, "Environment variable '" TAC_ENV_PORT \
		     "' must be a number.\n", NULL);
	free(host);
	return (TMR_INTERNAL);
    }

    /* find inet-address of host. first check dotted-decimal... */
    if ( (inaddr = inet_addr(host)) == INADDR_NONE ) {
	if ( (hep = gethostbyname(host)) == NULL ) {
	    sprintf(status, "meet %s lookup: ", host);
#ifdef __FreeBSD__
	    if ( h_errno != 0 )
		strcat(status, hstrerror(h_errno));
	    else
#endif
		strcat(status, strerror(errno));
	    folder_store(abc, TAC_STATUS, status, NULL);
	    free(host);
	    return (TMR_NOSITE);
	}
	else
	    inaddr = ((struct in_addr *)(hep->h_addr_list[0]))->s_addr;
    }
    free(host);

    /* set up address structure */
    memset(&addr, 0, sizeof(struct sockaddr_in));
    addr.sin_family = AF_INET;
    addr.sin_port = htons(port);
    addr.sin_addr.s_addr = inaddr;

    /* see if we have to encrypt or sign briefcase */
    if ( flags & (TM_CRPT|TM_SIGN) )
    {
	arc = archive(abc);
	bc = bc_create();
	folder_storeA(bc, TAC_AGENTID, folder_look(abc, TAC_AGENTID));
	folder_store(bc, TAC_CR_BC, arc, NULL);
	free(arc);

	if ( (syscab = cabinet_open(TAC_SYSTEM_CAB, FALSE)) ) {
	    if ( (data = folder_fetch(syscab, TAC_CRYPT_MD)) ) {
		folder_storeA(bc, TAC_CR_METHOD, data);
		free_copy(data);
	    }
	    cabinet_close(syscab);
	}
	else
	    folder_store(bc, TAC_CR_METHOD, DEFCRYPTMD, NULL);

	folder_store(bc, TAC_CR_SIGN, TAC_CR_BC, NULL);
	folder_store(bc, TAC_CR_ENCRYPT, TAC_CR_BC, NULL);
	folder_store(bc, TAC_CR_PASSPHRASE, PASSPHRASE, NULL);
	if ( meet(TAC_AG_CRYPT, bc, TM_LOCL|TM_SYNC) )
	{
	    folder_storeA(abc, TAC_STATUS, folder_look(bc, TAC_STATUS));
	    return (TMR_AG_DEAD);
	}
	folder_delete(bc, TAC_CR_SIGN);
	folder_delete(bc, TAC_CR_ENCRYPT);
	folder_delete(bc, TAC_CR_PASSPHRASE);
    }
    else
	bc = abc;

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if ( connect(sockfd, (struct sockaddr *) &addr, sizeof(addr)) == -1 )
    {
	sprintf(status, "meet connect error: %s", strerror(errno));
	folder_store(abc, TAC_STATUS, status, NULL);
	return (TMR_NO1SITE);
    }

    arcp = tac_read_bc_stream(sockfd);
#ifdef TM_DEBUG
    fprintf(stderr, "Connect request returned '%s'", arcp);
#endif

    /*
     * Got response from remote firewall to connect req. 
     * Simply check the integer value of the first word;
     * if it is 0 everything is ok, and we discard the
     * welcome/error message.
     */
    if ( isdigit(*arcp) ) {
	if ( (retval = atoi(arcp)) != 0 ) {
	    arc = strchr(arcp, ' ');
	    folder_store(abc, TAC_STATUS, ++arc, NULL);
	    close(sockfd);
	    free(arcp);
	    return (retval);
	}
	else
	    free(arcp);
    }
    else {
	folder_store(abc, TAC_STATUS, arcp, NULL);
	close(sockfd);
	free(arcp);
	return (TMR_INTERNAL);
    }

#ifdef TM_DEBUG
    fprintf(stderr, "Writing preamble: %d\n", flags);		    
#endif
    sprintf(status, "%d ", flags);
    write(sockfd, status, strlen(status));
#ifdef TM_DEBUG
    fprintf(stderr, "Writing bc\n---\n%s\n---\n", arc = archive(bc));
    free(arc);
#endif
    tac_write_bc_stream(sockfd, bc);

    arcp = tac_read_bc_stream(sockfd);
#ifdef TM_DEBUG
    fprintf(stderr, "Remote firewall replied: '%s'\n", arcp);
#endif
    if ( isdigit(*arcp) ) {
	if ( (retval = atoi(arcp)) != 0 ) {
	    arc = strchr(arcp, ' ') + 1;
	    folder_store(abc, TAC_STATUS, arc, NULL);
	    close(sockfd);
	    free(arcp);
	    return (retval);
	}
	else
	    free(arcp);
    }
    else {
	folder_store(abc, TAC_STATUS, arcp, NULL);
	close(sockfd);
	free(arcp);
	return (TMR_INTERNAL);
    }

    if ( flags & TM_SYNC ) {
	arcp = tac_read_bc_stream(sockfd);
	*(arc = strchr(arcp, ' ')) = '\0';
	arc++; 
	retval = atoi(arcp);
#ifdef TM_DEBUG
	fprintf(stderr,	"Returned bc\n===========\n%s\n===========\n", arc);
#endif
	unarchive(abc, arc);
	free(arcp);
    }
    else {
	if ( !(flags & TM_NLOG) &&
	     (outlog = cabinet_open(TAC_OUTLOG_CAB, TRUE)) ) {
	    arc = archive(bc);
	    folder_store(outlog, *folder_look(abc, TAC_AGENTID), arc, NULL);
	    free(arc);
	    cabinet_close(outlog);
	}
    }

    close(sockfd);

    return (retval);
}


string
tac_read_bc_stream( int fd )
{
    int	    n;
    int	    pos;
    int	    size;
    string  buf;

    if ( (buf = malloc(PAGE_SIZE)) == NULL )
	return (NULL);

    for ( size=PAGE_SIZE,pos=0; (n=read(fd, &buf[pos], size-pos-1)) > 0; )
    {
	if ( (pos+=n) == size-1 )
	    if ( (buf = realloc(buf, (size+=PAGE_SIZE))) == NULL )
		return (NULL);
	if ( !strncmp(&buf[pos-TM_BCEND_SIZE], TM_BCEND, TM_BCEND_SIZE) ) {
	    buf[pos-TM_BCEND_SIZE] = '\0';
	    return (buf);
	}
    }

    free(buf);

    return (NULL);
}


int
tac_write_bc_stream( int fd, briefcase bc )
{
    int	    n;
    string  buf;

    buf = archive(bc);
    if ( (n = write(fd, buf, strlen(buf))) != -1 )
	if ( write(fd, TM_BCEND, TM_BCEND_SIZE) == -1 )
	    n = -1;

    free(buf);

    return (n);
}


/*
 *	Utility functions used in this file.
 */
static int
local_meet( string contact, briefcase abc, int flags )
{
    int		      ofd;
    int		      retval;
    int		      p1fd[2];
    int		      p2fd[2];
#ifndef TM_DEBUG
    char	      errName[L_tmpnam];
#endif
    char	      outName[L_tmpnam];
    char	      status[TM_STAT_MSG_SIZE];
    pid_t	      ag_pid;
    pid_t	      child_pid;
    string	      arc;
    string	      cmdline;
    string	      tacomapath;
    struct stat	      afs;
    struct sigaction  act;

#ifdef TM_DEBUG
    fprintf(stderr, "Local %s meet with '%s'.\n",
	    flags&TM_SYNC ? "sync" : "async", contact);
#endif

    retval = TMR_OK;

    /* find command line and contact */
    if ( flags & TM_PATH ) {
	if ( (cmdline=strdup(contact)) == NULL )
	    return (-1);
	while ( (arc=strchr(contact, '/')) )
	    contact = ++arc;
    }
    else {
	if ( (tacomapath = getenv(TAC_ENV_PATH)) == NULL )
	    return (-1);
	if ( (cmdline = malloc(strlen(tacomapath)+strlen(contact)+
			       (tacomapath[strlen(tacomapath)-1]=='/'?1:2)+
			       strlen(TAC_SYSAGENTS))) == NULL )
	    return (-1);
	sprintf(cmdline, "%s%s" TAC_SYSAGENTS "%s", tacomapath,
		tacomapath[strlen(tacomapath)-1] == '/' ? "" : "/", contact );
    }

    /* check if contact exists, and that we have permission to run it */
    if ( stat(cmdline, &afs) == -1 ||
	 !(afs.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)) ) {
	sprintf(status, "Unknown agent '%s'.", contact);
	folder_store(abc, TAC_STATUS, status, NULL);
	free(cmdline);
	return (TMR_AG_NOTFOUND);
    }

    /* local_meet -> contact */
    if ( pipe(p1fd) == -1 ) {
	free(cmdline);
	return (-1);
    }

    /* local_meet <- contact */
    if ( flags & TM_SYNC ) {
	if ( pipe(p2fd) == -1 ) {
	    close(p1fd[0]);
	    close(p1fd[1]);
	    free(cmdline);
	    return (-1);
	}
    }
    else
	tmpnam(outName);

#ifndef TM_DEBUG
    /* create a tmp file for the stderr of the agent activated */
    tmpnam(errName);
#endif

    /* register signal handler to catch dead children */
    act.sa_handler = tac_sigchld;
    sigemptyset(&act.sa_mask);
    act.sa_flags = SA_NOCLDSTOP | SA_RESTART;
    if ( sigaction(SIGCHLD, &act, NULL) == -1 ) {
	fprintf(stderr, "meet(): could not catch dead children: %s\n",
		strerror(errno));
	exit(1);
    }

    if ( flags & TM_SYNC ) {
      /* want to wait for child */
      act.sa_handler = SIG_DFL;
      sigemptyset(&act.sa_mask);
      if ( sigaction(SIGCHLD, &act, NULL) == -1 ) {
	fprintf(stderr, "meet(): could not wait for child: %s\n",
		strerror(errno));
	exit(1);
      }
    }
    else {
      /* catch dead children */
      act.sa_handler = tac_sigchld;
      sigemptyset(&act.sa_mask);
      act.sa_flags = SA_NOCLDSTOP | SA_RESTART;
      if ( sigaction(SIGCHLD, &act, NULL) == -1 ) {
	fprintf(stderr, "meet(): could not catch dead children: %s\n",
		strerror(errno));
	exit(1);
      }
    }

    /* fork of a child to start agent */
    switch ( (child_pid = fork()) )
    {
      case -1:
	folder_store(abc, TAC_STATUS, "meet(): unable to fork.", NULL);
	retval =  TMR_INTERNAL;
	break;

      case 0:
	/* make contact's input come from pipe 1 */
	close(0);
	dup(p1fd[0]);
	close(p1fd[0]);
	close(p1fd[1]);

	/* make contact's output go to pipe 2 or to file */
	close(1);
	if ( flags & TM_SYNC ) {
	    dup(p2fd[1]);
	    close(p2fd[0]);
	    close(p2fd[1]);
	}
	else {
	    if ( open(outName, O_WRONLY | O_CREAT | O_TRUNC, 0600) == -1 )
		exit(1);	/* XXX - what should we really do here */
	}

#ifndef	TM_DEBUG
	/* make contact's error output go to file */
	close(2);
	open(errName, O_WRONLY|O_CREAT|O_TRUNC, 0600);
#endif

	/* We want to wait for child */
	act.sa_handler = SIG_DFL;
	sigemptyset(&act.sa_mask);
	if ( sigaction(SIGCHLD, &act, NULL) == -1 ) {
	  fprintf(stderr, "meet(): could not wait for children: %s\n",
		  strerror(errno));
	  exit(1);
	}

	/* execute contact agent */
	switch ( (ag_pid = vfork()) ) {
	  case -1:
	    folder_store(abc, TAC_STATUS, "meet(): unable to vfork.", NULL);
	    retval =  TMR_INTERNAL;
	    break;

	  case 0:
	    execl(cmdline, contact, NULL);
	    fprintf(stderr, "meet(): failed to start %s: %s\n",
		    cmdline, strerror(errno));
	    _exit(1);

	  default:
	    /* free resources not needed anymore */
	    free(cmdline);
	    close(0);
	    close(1);
#ifndef	TM_DEBUG
	    close(2);
#endif

	    /* wait for contact to die */
	    waitpid(ag_pid, &retval, 0);

	    /* now check what happened to contact */
	    if ( WIFEXITED(retval) ) {
		retval = WEXITSTATUS(retval);
		if ( retval !=0 && !(flags & (TM_SYNC|TM_IERR)) ) {
		    /* contact reports error */
		  arc = NULL;
		  if ( (ofd = open(outName, O_RDONLY)) != -1 ) {
		    arc = tac_read_bc_stream(ofd);
		    close(ofd);
		  }
		  if (arc) {
		    unarchive(abc, arc);
		    free(arc);
		  }
		  else {
		    sprintf(status, "%s terminated abnormally (status=%d)",
			    contact, retval);
#ifndef TM_DEBUG
		    tac_read_stderr(abc, errName, status);
#else
		    tac_read_stderr(abc, NULL, status);
#endif
		  }
		}
		/*if ( retval != 0 )
		  retval = TMR_AG_DEAD;*/
	    }
	    else {
	      if ( !(flags & (TM_SYNC|TM_IERR) ) ) {
		sprintf(status, "%s %s by signal %d", contact,
			WIFSIGNALED(retval) ? "terminated" : "stopped",
			WIFSIGNALED(retval) ? WTERMSIG(retval)
			: WSTOPSIG(retval) );
#ifndef TM_DEBUG
		tac_read_stderr(abc, errName, status);
#else
		tac_read_stderr(abc, NULL, status);
#endif
	      }
	      retval = 128 + (WIFSIGNALED(retval) ? WTERMSIG(retval)
			: WSTOPSIG(retval));
	    }

	    /* create error report */
	    if ( (retval != 0) && !(flags & (TM_SYNC|TM_IERR)) )
		tac_create_error_rep(abc);

	    if( !(flags & TM_SYNC) ) {
		unlink(outName);
#ifndef	TM_DEBUG
		unlink(errName);
#endif
	    }
	}

	_exit(retval);
	break;

      default:
	/* close read part of local_meet's write pipe */
	close(p1fd[0]);

	/* close write part of local_meet's read pipe,
	 * that is, if we're not performing a syncronous meet. */
	if ( flags & TM_SYNC )
	    close(p2fd[1]);

	/* ignore PIPE signal before writing to pipe,
	 * in case no-one is listning. */
	signal(SIGPIPE, SIG_IGN);
	tac_write_bc_stream(p1fd[1], abc);
	signal(SIGPIPE, SIG_DFL);
	close(p1fd[1]);

	if ( flags & TM_SYNC ) {
	    arc = tac_read_bc_stream(p2fd[0]);

	    /* wait for child */
	    waitpid(child_pid, &retval, 0);

	    close(p2fd[0]);

#ifdef TM_DEBUG
	    fprintf(stderr,
		    "Returned bc\n===========\n%s\n===========\n", arc);
#endif
	    if ( WIFEXITED(retval) ) {
		retval = (int)(char) WEXITSTATUS(retval);
		if ( retval < -96 ) {
		    sprintf(status, "%s stopped by signal %d",
			    contact, 128+retval);
#ifndef TM_DEBUG
		    tac_read_stderr(abc, errName, status);
#else
		    tac_read_stderr(abc, NULL, status);
#endif
		    retval = TMR_AG_DEAD;
		}
		else {
		    if ( arc ) {
			unarchive(abc, arc);
			free(arc);
		    }
		    else {
			sprintf(status, "%s terminated abnormally", contact);
#ifndef TM_DEBUG
			tac_read_stderr(abc, errName, status);
#else
			tac_read_stderr(abc, NULL, status);
#endif
			retval = TMR_AG_DEAD;
		    }
		}
	    }
	    else {
		sprintf(status, "meet() failure: child aborted (sig %d).",
			WIFSIGNALED(retval)?WTERMSIG(retval):WSTOPSIG(retval));
		folder_store(abc, TAC_STATUS, status, NULL);
		retval = TMR_INTERNAL;
	    }

#ifndef TM_DEBUG
	    unlink(errName);
#endif
	}
	break;
    }

    /* change to old cwd and free resources */
    /*fchdir(cwdfd);*/
    free(cmdline);

    return (retval);
}


static void
tac_sigchld( int signo )
{
    /* non-blocking wait for any child, discard status */
    waitpid(-1, NULL, WNOHANG);
}


static void
tac_read_stderr( briefcase bc, string errName, string errmsg )
{
    int		 len;	
    int		 efd;
    struct stat	 efs;
    string	 etxt;

    if ( errName && (efd = open(errName, O_RDONLY)) != -1 )
    {
	fstat(efd, &efs);
	if ( !(etxt=malloc(efs.st_size + strlen(errmsg) + 3)) )
	    folder_store(bc, TAC_STATUS, errmsg, NULL);
	else {
	    strcpy(etxt, errmsg);
	    strcat(etxt, ": ");
	    len = strlen(etxt);
	    etxt[len+read(efd, &etxt[len], efs.st_size)]=0;
	    folder_store(bc, TAC_STATUS, etxt, NULL);
	    free(etxt);
	}
	close(efd);
    }
    else
	folder_store(bc, TAC_STATUS, errmsg, NULL);
}


static void
tac_create_error_rep( briefcase bc )
{
    char	host[MAXHOSTNAMELEN];
    time_t	clock;
    string	arc;
    string*	data;
    briefcase	errbc;

    /* no use returning info if no-one to return to! */
    if ( ((data = folder_look(bc, TAC_RETURNHOST)) == NULL) ||
	 (*data == NULL) || (**data == '\0') ) 
      return;

    /* make sure there is a TAC_STATUS folder */
    if ( folder_look(bc, TAC_STATUS) == NULL )
	folder_store(bc, TAC_STATUS, "", NULL);

    /* collect info on agent */
    arc = archive(bc);
    clock = time(NULL);
    gethostname(host, MAXHOSTNAMELEN);

    /* create error report */
    errbc = bc_create();
    folder_store(errbc, TAC_R_FROM,   host,			     NULL);
    folder_store(errbc, TAC_R_BC,     arc,			     NULL);
    folder_store(errbc, TAC_R_DATE,   ctime(&clock),		     NULL);
    folder_storeA(errbc,TAC_R_ID,     folder_look(bc, TAC_AGENTID        ));
    folder_storeA(errbc,TAC_R_STATUS, folder_look(bc, TAC_STATUS         ));
    folder_store(errbc, TAC_R_LEVEL,  "failure",		             NULL);
    folder_storeA(errbc,TAC_HOST,     data                               );

    /* meet with report agent */
   meet(TAC_AG_REPORT, errbc, TM_IERR | TM_NLOG);

    /* free resources */
    free(arc);
    bc_discard(errbc);
}
