/*******************************************************************************
 *
 * $Source: /users/studenter/nilss/source/RCS/tacoma.h,v $
 * $Header: tacoma.h,v 1.5 96/03/10 18:53:54 nilss Exp $
 * $Author: nilss $
 * $Date: 96/03/10 18:53:54 $
 * Language: C
 * Status: Experimental
 *
 * (c) Copyright 1996, TACOMA, all rights reserved.
 *
 ******************************************************************************/
#ifndef	TACOMA_H
#define	TACOMA_H

/*
 * Directories used by Tacoma.
 */
#define	TAC_BIN		"bin/"
#define	TAC_CABINETS	"cabinets/"
#define	TAC_SYSAGENTS	"sysagents/"

/*
 * Cabinets used by Tacoma.
 */
#define	TAC_INLOG_CAB	"inlog"
#define	TAC_OUTLOG_CAB	"outlog"

#define	TAC_SYSTEM_CAB	"system"
/* folders in system cabinet */
#define	TAC_AUTOHOSTS	"AutoAcceptHost"
#define	TAC_ACCEPTFROM	"AcceptFrom"
#define	TAC_STARTUP_AG	"StartupAgents"
#define	TAC_CRYPT_MD	"DefCryptMethod"

/*
 * Names of system agents used in Tacoma.
 */
#define	TAC_AG_CRYPT	"ag_crypt"
#define	TAC_AG_REPORT	"ag_report"

/*
 * Folder names used by system agents in Tacoma.
 */
#define	TAC_AGENTID	"AGENTID"
#define	TAC_CONTACT	"CONTACT"
#define	TAC_STATUS	"STATUS"
#define	TAC_HOST	"HOST"
#define	TAC_RETURNHOST	"RETURNHOST"

/* folder names used by ag_crypt */
#define	TAC_CR_BC	"CR_BC"
#define	TAC_CR_BCSIGN	"CR_BC.sign"
#define	TAC_CR_METHOD	"CR_METHOD"
#define	TAC_CR_SIGN	"CR_SIGN"
#define	TAC_CR_ENCRYPT	"CR_ENCRYPT"
#define	TAC_CR_DECRYPT	"CR_DECRYPT"
#define	TAC_CR_PASSPHRASE "CR_PASSPHRASE"

/* folder names used by ag_report */
#define	TAC_R_FROM	"R_FROM"
#define	TAC_R_BC	"R_BC"
#define	TAC_R_DATE	"R_DATE"
#define	TAC_R_ID	"R_ID"
#define	TAC_R_STATUS	"R_STATUS"
#define	TAC_R_LEVEL	"R_LEVEL"

/*
 * Environment variables used by Tacoma.
 */
#define	TAC_ENV_PATH	"TACOMAPATH"
#define	TAC_ENV_PORT	"TACOMAPORT"

/*
 * TACOMA notification message.
 */
#define	TAC_NOTIFY	"Make a note of error, agents, system agents, logs " \
			"and software version used.\nTry to reproduce error" \
			", then contact the TACOMA project at:\n" \
			"tacoma@cs.uit.no"

/*
 * Internal magic numbers used by Tacoma.
 * Don't change unless you're pretty damned sure of what you're doing.
 */
#define	TAC_BC_HASHSIZE	 29
#define	TAC_ID_SIZE	129

/*
 * Deallocates a NULL-terminated array of strings.
 */
#define free_copy(sp)	if (sp) {		\
			    string* str_p = sp;	\
			    while ( *str_p )	\
				free(*str_p++);	\
			    free(sp);		\
			}


/*
 * New types and structs used in Tacoma.
 */
#ifndef FALSE
#define	FALSE	0
#define	TRUE	1
#endif

typedef char*	string;

typedef struct _folder
{
    string	    name;
    string*	    contents;
    struct _folder* next;
    struct _folder* prev;
} folder_rec, *folder;

typedef struct _briefcase
{
    string  name;
    folder* f;
} briefcase_rec, *briefcase;

typedef	briefcase_rec	cabinet_rec;
typedef briefcase	cabinet;

typedef enum _direction
{
    TAC_LEFT  = -1,
    TAC_RIGHT =  1
} direction;

/* flags used in cabinet_open */
#define	TC_NONE		0
#define	TC_CREAT	(1 << 0)


/*
 * Functions for creating and manipulating briefcases. (In tacoma.c)
 */
briefcase bc_create( void );
void	  bc_discard( briefcase );
string*	  bc_list( briefcase );
string	  archive( briefcase );
briefcase unarchive( briefcase, string );

/*
 * Functions for creating and manipulating cabinets. (In tacoma.c)
 */
cabinet	cabinet_open( string, int );
void	cabinet_close( cabinet );
void	cabinet_discard( cabinet );
string*	cabinet_list( cabinet );

/*
 * Functions for creating and manipulating folders. (In tacoma.c)
 */
int	folder_storeA( briefcase, string, string* );
int	folder_store( briefcase, string, ... );
string*	folder_fetch( briefcase, string );
string*	folder_look( briefcase, string );
void	folder_delete( briefcase, string );
string	folder_rotate( briefcase, string, direction, int );
string	folder_shiftA( briefcase, string, direction, int, string* );
string	folder_shift( briefcase, string, direction, int, ... );

#define folder_pop(x, y) folder_shift(x, y, TAC_LEFT, 1, 0L)
#define folder_add(x, y, z) folder_shift(x, y, TAC_LEFT, 0, z, 0L)
#define folder_push(x, y, z) folder_shift(x, y, TAC_RIGHT, 0, z, 0L)
#define folder_top(x, y) folder_shift(x, y, TAC_LEFT, 0, 0L)

/*
 * Function to arrange a meet with another agent. (In meet.c)
 */
int	meet( string, briefcase, int );


#endif	/* TACOMA_H */

