#include <stdio.h>
#include <unistd.h>
#include "scheme.h"
#include "tacoma.h"
#include "meet.h"
#include "tacoma_elk.h"


/*
 *   Object that will hold the incoming briefcase
 */
Object V_Inc_Bc;


/*
 *   Evaluate scheme-expressions.
 */
void eval_string( expr )
char *expr;
{
    Object port;
    GC_Node;

    /* Create a string-port containing the expression */
    port = P_Open_Input_String( Make_String( expr, strlen(expr) ) );

    /* We have to make port reachable by the GC since
       Load_Source_Port may trigger garbage collection */
    GC_Link( port );
    Load_Source_Port( port );
    GC_Unlink;

    /* Close string-port again */
    (void) P_Close_Input_Port( port );
}


/*
 *   Main should be called with no arguments, and a archived briefcase
 *   should be arriving on the standard input.
 */
int main( argc, argv )
int argc;
char *argv[];
{
    char *arch, **code;
    char *op, *tp, *np;

    /*
     * Elk_Init needs to know where to find ag_elk.
     * Ie., we have to update PATH. 
     *
     *   PATH=${PATH}:${TACOMAPATH}/sysagents
     */
    op = (char*) getenv( "PATH" );
    tp = (char*) getenv( TAC_ENV_PATH );
    np = (char*) malloc( 5 + strlen(op) + 1 + strlen(tp) + 1 +
			 strlen(TAC_SYSAGENTS) + 1 );
    sprintf( np, "PATH=%s:%s%s%s", op, tp, 
	     tp[strlen(tp)-1] == '/' ? "" : "/", TAC_SYSAGENTS );
    putenv( np );

    /* Start the scheme interpreter */
    Elk_Init( argc, argv, 1, 0 );

    /* Create incoming briefcase */
    Var_Set( V_Inc_Bc, P_Bc_Create() );

    /* Get the archived briefcase from stdin and unarchive it */
    arch = tac_read_bc_stream( STDIN_FILENO );
    unarchive( Get_Briefcase( Var_Get(V_Inc_Bc) ), arch );
    free( arch );

    /* Evaluate all elements in the SCMCODE-folder */
    code = folder_fetch( Get_Briefcase( Var_Get(V_Inc_Bc) ), "SCMCODE" );
    while( *code ) eval_string( *code++ );

    /* Evaluate the leftmost element in the SCMEVAL-folder,
       and rotate the folder */
    eval_string( folder_rotate( Get_Briefcase( Var_Get(V_Inc_Bc) ), 
				"SCMEVAL", TAC_LEFT, 1 ) );

    /* Archive briefcase and write it to stdout */
    tac_write_bc_stream( STDOUT_FILENO, 
			 Get_Briefcase( Var_Get(V_Inc_Bc) ) );

    exit(0);
}


/*
 *  Define a variable, `inc-bc', that will hold the incoming briefcase.
 */
void elk_init_ag_elk()
{
    Define_Variable( &V_Inc_Bc, "inc-bc", Null );
}



