
#include "scheme.h"
#include "tacoma.h"
#include "meet.h"
#include "tacoma_elk.h"


/*
 *   Variable to hold the latest error-code returned from meet.
 */
Object V_Meet_Error;


/*
 *   Flags that may be given to the meet-primitive.
 */
static SYMDESCR meet_syms[] = {
    { "sync",		TM_SYNC },
    { "rpc",		TM_SYNC },
    { "local",		TM_LOCL },
    { "nomerge",	TM_NMRG },
    { "ignerr",		TM_IERR },
    { "crypt",		TM_CRPT },
    { "sign",		TM_SIGN },
    { "path",		TM_PATH },
    { "nolog",		TM_NLOG },
    { 0,		0 }
};


/*
 *   Values returned by the meet-primitive.
 *   (An integer < 0 may also be returned).
 */
static SYMDESCR meet_return[] = {
    { "OK",		TMR_OK },
    { "notreached",	TMR_NO1SITE },
    { "refused",	TMR_REFUSED },
    { "nosite",		TMR_NOSITE },
    { "ag-dead",	TMR_AG_DEAD },
    { "ag-notfound",	TMR_AG_NOTFOUND },
    { "internal",	TMR_INTERNAL }
};


/*
 *   (meet ag bc [flags])
 *
 *   Meet agent, ag, with briefcase, bc.  Flags that may be specified
 *   include:  sync (rpc), local, nomerge, ignerr, crypt, sign,
 *   path and nolog.  They are specified as a list of symbols.  Eg:
 *
 *	(meet "ag_scheme" bc '(sync nomerge))
 */
Object P_Meet( argc, argv )
int argc;
Object *argv;
{
    char *agent;
    briefcase bc;
    int flags = 0, ret;

    /* Get agent name and briefcase */
    agent = Get_Strsym( argv[0] );
    bc = Get_Briefcase( argv[1] );

    /* Decode symbols */
    if ( argc == 3 )
	flags = Symbols_To_Bits( argv[2], 1, meet_syms );

    /* Meet with agent */
    ret = meet( agent, bc, flags );

    /* Store result for later use */
    Var_Set( V_Meet_Error, (ret < 0 ? Make_Integer( ret ) :
			    Bits_To_Symbols( ret, 0, meet_return )) );

    return ret == TMR_OK ? True : False;
}


/*
 *   Define primitive meet, and the variable meet-error.
 */
void elk_init_tac_meet()
{
    Define_Variable( &V_Meet_Error, "meet-error", Null );
    Define_Primitive( P_Meet, "meet", 2, 3, VARARGS );
}


