#ifdef __cplusplus
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include <tacoma.h>
#include <meet.h>
#ifdef __cplusplus
}
#endif


typedef struct _briefcase  Briefcase;
typedef	Briefcase	   Cabinet;


MODULE = Tacoma		PACKAGE = Tacoma

PROTOTYPES: DISABLE


MODULE = Tacoma		PACKAGE = Briefcase


void
new(self)
	SV *	   self
	CODE:
	char *	   class;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		class = HvNAME(SvSTASH(SvRV(self)));
	    else
		croak("self is not of type Briefcase");
	}
	else if ( SvPOK(self) ) {
	    class = SvPV(self, na);
	    if ( strNE(class, "Briefcase") )
		croak("self if not Briefcase");
	}
	else
	    croak("self is not package name or object reference");

	ST(0) = sv_newmortal();
	if ( (bc = bc_create()) )
	    sv_setref_pv(ST(0), class, (void*) bc);

void
DESTROY(self)
	SV *	   self
	CODE:
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	bc_discard(bc);

void
list(self)
	SV *	   self
	PPCODE:
	int	   count;
	string *   str_p;
	string *   strings;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}

	if ( (strings = bc_list(bc)) ) {
	    for ( str_p=strings; *str_p; str_p++ )
		count++;
	    EXTEND(sp, count);
	    for ( str_p=strings; *str_p; str_p++ )
		PUSHs(sv_2mortal(newSVpv(*str_p, 0)));
	    free_copy(strings);
	}
	else {
	    /* empty list */
	}


void
archive(self)
	SV *	   self
	CODE:
	char *	   arc;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	ST(0) = sv_newmortal();
	if ( (arc = archive(bc)) ) {
	    sv_setpv(ST(0), arc);
	    free(arc);
	}

void
unarchive(self, arc)
	SV *	   self
	char *	   arc
	CODE:
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	ST(0) = sv_newmortal();
	if ( (unarchive(bc, arc)) )
	    ST(0) = self;

int
store(self, name, ...)
	SV *	   self
	char *	   name
	CODE:
	int	   idx;
	string *   strings;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	if ( (strings = malloc((items-1)*sizeof(string))) == NULL )
	    RETVAL = -1;
	else {
	    for ( idx=0; idx < (items-2); idx++ )
		strings[idx] = SvPV(ST(idx+2), na);
	    strings[items-2] = NULL;
	    RETVAL = folder_storeA(bc, name, strings);
	    free(strings);
	}

string*
fetch(self, name)
	SV *	   self
	char *	   name
	PPCODE:
	int	   count;
	string *   str_p;
	string *   strings;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}

	if ( (strings = folder_fetch(bc, name)) ) {
	    for ( str_p=strings; *str_p; str_p++ )
		count++;
	    EXTEND(sp, count);
	    for ( str_p=strings; *str_p; str_p++ )
		PUSHs(sv_2mortal(newSVpv(*str_p, 0)));
	    free_copy(strings);
	}
	else {
	    /* empty list */
	}

void
delete(self, name)
	SV *	   self
	char *	   name
	CODE:
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	folder_delete(bc, name);

void
rotate(self, name, direction, count)
	SV *	   self
	char *	   name
	char *	   direction
	int	   count
	CODE:
	int	   dir;
	string	   str;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	if ( strEQ(direction, "left") )
	    dir = TAC_LEFT;
	else if ( strEQ(direction, "right") )
	    dir = TAC_RIGHT;
	else
	    croak("direction must be 'left' or 'right'");

	ST(0) = sv_newmortal();
	if ( (str = folder_rotate(bc, name, dir, count)) ) {
	    sv_setpv(ST(0), str);
	    free(str);
	}

void
shift(self, name, direction, count, ...)
	SV *	   self
	char *	   name
	char *	   direction
	int	   count
	CODE:
	int	   dir;
	int	   idx;
	string	   str;
	string *   strings;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	if ( strEQ(direction, "left") )
	    dir = TAC_LEFT;
	else if ( strEQ(direction, "right") )
	    dir = TAC_RIGHT;
	else
	    croak("direction must be 'left' or 'right'");

	ST(0) = sv_newmortal();
	if ( items > 3 ) {
	    if ( (strings = malloc((items-2)*sizeof(string))) ) {
		for ( idx=0; idx < (items-3); idx++ )
		    strings[idx] = SvPV(ST(idx+3), na);
		strings[items-3] = NULL;
		if ( (str = folder_shiftA(bc, name, dir, count, strings)) ) {
		    sv_setpv(ST(0), str);
		    free(str);
		}
		free(strings);
	    }
	}
	else {
	    if ( (str = folder_shiftA(bc, name, dir, count, NULL)) ) {
		sv_setpv(ST(0), str);
		free(str);
	    }
	}

void
pop(self, name)
	SV *	   self
	char *	   name
	CODE:
	string	   str;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	ST(0) = sv_newmortal();
	if ( (str = folder_shift(bc, name, TAC_LEFT, 1, NULL)) ) {
	    sv_setpv(ST(0), str);
	    free(str);
	}

void
push(self, name, str)
	SV *	   self
	char *	   name
	char *	   str
	CODE:
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	folder_shift(bc, name, TAC_RIGHT, 0, str, NULL);

void
top(self, name)
	SV *	   self
	char *	   name
	CODE:
	string	   str;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	ST(0) = sv_newmortal();
	if ( (str = folder_shift(bc, name, TAC_LEFT, 0, NULL)) ) {
	    sv_setpv(ST(0), str);
	    free(str);
	}

void
add(self, name, str)
	SV *	   self
	char *	   name
	char *	   str
	CODE:
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}
	folder_shift(bc, name, TAC_LEFT, 0, str, NULL);


MODULE = Tacoma		PACKAGE = Cabinet
void
new(self, name, ...)
	SV *	 self
	char *	 name
	CODE:
	int	 flag;
	char *	 class;
	cabinet	 cab;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Cabinet") )
		class = HvNAME(SvSTASH(SvRV(self)));
	    else
		croak("self is not of type Cabinet");
	}
	else if ( SvPOK(self) ) {
	    class = SvPV(self, na);
	    if ( strNE(class, "Cabinet") )
		croak("self if not Cabinet");
	}
	else
	    croak("self is not package name or object reference");

	if ( items > 3 )
	    croak("Cabinet::new() takes only one optional argument");

	flag = TC_NONE;
	if ( (items > 2) && strEQ(SvPV(ST(2), na), "create") )
	    flag = TC_CREAT;

	ST(0) = sv_newmortal();
	if ( (cab = cabinet_open(name, flag)) )
	    sv_setref_pv(ST(0), class, (void*) cab);

void
DESTROY(self)
	SV *	 self
	CODE:
	cabinet	 cab;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Cabinet") )
		cab = (cabinet) SvIV(SvRV(self));
	    else
		croak("self is not of type Cabinet");
	}
	cabinet_close(cab);

void
discard(self)
	SV *	 self
	CODE:
	cabinet	 cab;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Cabinet") )
		cab = (cabinet) SvIV(SvRV(self));
	    else
		croak("self is not of type Cabinet");
	}
	cabinet_discard(cab);
	sv_setsv(SvRV(self), &sv_undef);
	SvREFCNT_dec(self);
	self = &sv_undef;
	OUTPUT:
	self

void
list(self)
	SV *	  self
	PPCODE:
	int	  count;
	string *  str_p;
	string *  strings;
	cabinet	  cab;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Cabinet") )
		cab = (cabinet) SvIV(SvRV(self));
	    else
		croak("self is not of type Cabinet");
	}

	if ( (strings = cabinet_list(cab)) ) {
	    for ( str_p=strings; *str_p; str_p++ )
		count++;
	    EXTEND(sp, count);
	    for ( str_p=strings; *str_p; str_p++ )
		PUSHs(sv_2mortal(newSVpv(*str_p, 0)));
	    free_copy(strings);
	}
	else {
	    /* empty list */
	}

void
delete(self, name)
	SV *	 self
	char *	 name
	CODE:
	cabinet  cab;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Cabinet") )
		cab = (cabinet) SvIV(SvRV(self));
	    else
		croak("self is not of type Cabinet");
	}
	folder_delete(cab, name);

int
store(self, name, ...)
	SV *	  self
	char *	  name
	CODE:
	int	  idx;
	string *  strings;
	cabinet	  cab;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Cabinet") )
		cab = (cabinet) SvIV(SvRV(self));
	    else
		croak("self is not of type Cabinet");
	}
	if ( (strings = malloc((items-1)*sizeof(string))) == NULL )
	    RETVAL = -1;
	else {
	    for ( idx=0; idx < (items-2); idx++ )
		strings[idx] = SvPV(ST(idx+2), na);
	    strings[items-2] = NULL;
	    RETVAL = folder_storeA(cab, name, strings);
	    free(strings);
	}

string*
fetch(self, name)
	SV *	  self
	char *	  name
	PPCODE:
	int	  count;
	string *  str_p;
	string *  strings;
	cabinet	  cab;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Cabinet") )
		cab = (cabinet) SvIV(SvRV(self));
	    else
		croak("self is not of type Cabinet");
	}

	if ( (strings = folder_fetch(cab, name)) ) {
	    for ( str_p=strings; *str_p; str_p++ )
		count++;
	    EXTEND(sp, count);
	    for ( str_p=strings; *str_p; str_p++ )
		PUSHs(sv_2mortal(newSVpv(*str_p, 0)));
	    free_copy(strings);
	}
	else {
	    /* empty list */
	}


MODULE = Tacoma		PACKAGE = Tacoma

int
meet(agent, self, ...)
	char *	   agent
	SV *	   self
	CODE:
	int	   idx;
	int	   flag;
	briefcase  bc;

	if ( sv_isobject(self) ) {
	    if ( sv_isa(self, "Briefcase") )
		bc = (briefcase) SvIV(SvRV(self));
	    else
		croak("self is not of type Briefcase");
	}

	flag = 0;
	if ( items > 2 )
	    for ( idx = 2; idx < items; idx++ ) {
		if ( strEQ(SvPV(ST(idx), na), "sync") ) {
		    if ( flag & TM_SYNC )
			croak("'sync' (or 'rpc') specified twice");
		    flag |= TM_SYNC;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "rpc") ) {
		    if ( flag & TM_SYNC )
			croak("'sync' (or 'rpc') specified twice");
		    flag |= TM_SYNC;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "local") ) {
		    if ( flag & TM_LOCL )
			croak("'local' specified twice");
		    flag |= TM_LOCL;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "nomerge") ) {
		    if ( flag & TM_NMRG )
			croak("'nomerge' specified twice");
		    flag |= TM_NMRG;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "ignerr") ) {
		    if ( flag & TM_IERR )
			croak("'ignerr' specified twice");
		    flag |= TM_IERR;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "crypt") ) {
		    if ( flag & TM_CRPT )
			croak("'crypt' specified twice");
		    flag |= TM_CRPT;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "sign") ) {
		    if ( flag & TM_SIGN )
			croak("'sign' specified twice");
		    flag |= TM_SIGN;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "path") ) {
		    if ( flag & TM_PATH )
			croak("'path' specified twice");
		    flag |= TM_PATH;
		    continue;
		}
		if ( strEQ(SvPV(ST(idx), na), "nolog") ) {
		    if ( flag & TM_NLOG )
			croak("'nolog' specified twice");
		    flag |= TM_NLOG;
		    continue;
		}
		croak("unknown flag");
	    }

	if ( (flag & TM_NMRG) && !(flag & TM_SYNC) )
	    croak("'nomerge' can only be specified with 'sync'");

	if ( (flag & TM_IERR) && (flag & TM_SYNC) )
	    croak("'ignerr' and 'sync' are mutually exclusive");

	RETVAL = meet(agent, bc, flag);
