/*
 * This is an example of how to make a C version of the ag_python
 * agent. The incoming briefcase is aptly created as a briefcase
 * object called 'incbc' in the interpreter namespace.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/param.h>

#include "Python.h"
#include "tacoma.h"
#include "meet.h"

/* Used for temporary storage of code-folder-contents */
static char *tmpdir = "/tmp";

/* This function is called by the interpreter to get its own name */
static char *argv0;
char *getprogramname()
{
      return argv0;
}

/*
 * Main, invocation point for ag_python
 */
int main( argc, argv )
int argc;
char *argv[];
{
    briefcase bc;
    char *arch, **code, fname[MAXPATHLEN];
    pid_t mypid;
    FILE *fp;

    /* Import Tacoma module and create the incbc-object */
    Py_Initialize();
    PyRun_SimpleString( "import tacoma" );
    PyRun_SimpleString( "incbc = tacoma.bc_create()" );

    /* Make 'incbc' point to the incoming briefcase */
    arch = tac_read_bc_stream( STDIN_FILENO );
    bc = bc_create();
    unarchive( bc, arch );
    tietolocal( bc );
    free( arch );

    /* This method should never be used except from here */
    PyRun_SimpleString( "incbc.locbc()" );

    /* Create a codefile from the PYCODE folder-contents */
    mypid = getpid();
    sprintf( fname, "%s/pycode%d", tmpdir, mypid );
    if ( (fp = fopen(fname, "w")) == NULL )
	exit( -1 );
    code = folder_fetch( bc, "PYCODE" );
    while( *code ) fprintf( fp, *code++ );
    fclose( fp );

    /* Now execute the commands in the file */
    if ( (fp = fopen(fname, "r")) == NULL )
	exit( -1 );
    PyRun_SimpleFile( fp );
    fclose( fp );
    unlink( fname );

    /* Archive briefcase and write it to stdout */
    tac_write_bc_stream( STDOUT_FILENO, bc );

    Py_Exit( 0 );
    /*NOTREACHED*/
}
