################################################################################
#
# $Source: /users/studenter/nilss/source/RCS/tacoma.tcl,v $
# $Header: tacoma.tcl,v 1.30 96/02/29 13:41:30 nilss Exp $
# $Author: nilss $
# $Date: 96/02/29 13:41:30 $ 
# Language: Tcl
# Status: Experimental
#
# (c) Copyright 1994, TACOMA, all rights reserved.
#
################################################################################
set ag_idnr 0

proc folder_fetch {bc foldername} {
    upvar $bc loc_bc
    if {[info exists loc_bc($foldername)] == 1} {
	return $loc_bc($foldername)
    } else {
	return ""
    }
}

proc folder_delete {bc foldername} {
    upvar $bc loc_bc
    if {[info exists loc_bc($foldername)] == 1} {
	unset loc_bc($foldername)
    }
}

proc folder_store {bc foldername args} {
    upvar $bc loc_bc
    if { [llength $args] > 1 } {
	set loc_bc(*\#$foldername) "1"
	set loc_bc($foldername) $args
    } else {
	set loc_bc($foldername) [lindex $args 0]
    }
}


proc archive {bc args} {
    upvar $bc loc_bc
    set list ""
    if { $args != "" } {
	set folders $args
    } else {
	set folders [array names loc_bc]
    }
    foreach folder $folders {
	if {[info exists loc_bc($folder)]} {
	    if {[string match \*\#* $folder]} {
		if {[info exists loc_bc([string range $folder 2 end])]} {
		    lappend list $folder $loc_bc($folder)
		}    
	    } else {
		lappend list $folder $loc_bc($folder)
	    }
	}
    }
    return $list
}

proc unarchive {bc bc_arc} {
    upvar $bc loc_bc
    for {set i 0} {$i < [llength $bc_arc]} {incr i 2} {
	set data [lindex $bc_arc [expr $i+1]]
	regsub -all {\\\\} $data "\\" data
	regsub -all {\\\{} $data "{" data
	regsub -all {\\\}} $data "}" data
	set loc_bc([lindex $bc_arc $i]) $data
    }
}

proc bc_create {db} {
    upvar $db loc_db
    global ag_idnr
    set loc_db(AGENTID) [exec date -u +%j%H%M%S][pid][incr ag_idnr 1] 
}

proc bc_discard {db} {
    upvar $db loc_db
    unset loc_db
}

proc bc_list {bc} {
    upvar $bc loc_bc
    set bcl ""
    foreach folder [array names loc_bc] {
	if {![string match {\**} $folder]} {
	    lappend bcl $folder
	}
    }
    return $bcl
}

proc cabinet_create {cabinet} {
    global env
    if { ![file isdirectory $env(TACOMAPATH)/cabinets] } {
	error "TACOMA filesystem corrupt: No cabinets can be found."
    }
    if { ![file isdirectory $env(TACOMAPATH)/cabinets/$cabinet] } {
	exec mkdir $env(TACOMAPATH)/cabinets/$cabinet
    }
    return $cabinet
}

proc cabinet_discard {cabinet} {
    global env
    catch {exec rm -r $env(TACOMAPATH)/cabinets/$cabinet}
    return $cabinet
}

proc cabinet_list {cabinet} {
    global env
    set cwd [pwd]
    cd $env(TACOMAPATH)/cabinets/$cabinet
    if { [catch { glob * } rc] } {
	set rc ""
    }
    cd $cwd
    return $rc
}

proc cabinet_open {cabinet} {
    global env
    if { ![file isdirectory $env(TACOMAPATH)/cabinets/$cabinet] } {
	error "TACOMA filesystem corrupt: Cabinet '$cabinet' cannot be located."
    }
    return $cabinet
}

proc cabinet_close {cabinet} {
# Not implemented !!!!!!!!!
}

proc cabinet_fetch {cabinet foldername} {
    global env
    set err [catch {open $env(TACOMAPATH)/cabinets/$cabinet/$foldername r} file]
    if { !$err } {
	set rc [read -nonewline $file]
	close $file
	return $rc
    } 
    return ""
}

proc cabinet_store {cabinet foldername data} {
    global env
    set err [catch {open $env(TACOMAPATH)/cabinets/$cabinet/$foldername w} file]
    if { !$err } {
	puts $file $data
	close $file
	return $data
    } 
    error "TACOMA filesystem corrupt: Folder '$foldername' cannot be written."
}

proc cabinet_delete {cabinet foldername} {
    global env
    set err [catch {exec rm $env(TACOMAPATH)/cabinets/$cabinet/$foldername}]
    if { !$err } {
	return $foldername
    }
    error "TACOMA filesystem corrupt: Folder '$cabinet/$foldername' couldn't be deleted."
}










